% This example shows the application of the automatic differentiation
% to the calculation of activities in the binary system
function ex2
x2 = (0:0.01:1)';
x2(x2 == 0) = 0.001;
x2(x2 == 1) = 1 - 0.001;
x2 = DualNumber.CreateXValue(x2);
% Calculate Gibbs energy
R = 8.314;
T = 298.15;
xmat_ = [1-x2';x2']';
xmat = [1-x2 x2];
max(max(real(xmat - xmat_)))
G = R*T*sum(xmat .* log(xmat), 2) + x2.*(1-x2).*(1000 + 4000*x2);
% Calculate activity coefficients by automatic differentation
lna1 = real(G) - real(x2) .* imag(G);
lna2 = real(G) + (1 - real(x2)) .* imag(G);
% Show the results
close all;
plot(real(x2), real(G), 'LineWidth', 2);
xlabel('x_2', 'FontSize', 14);
ylabel('\Delta_{mix}G, J/mol', 'FontSize', 14);
figure;
% a) automatic differentiation
plot(real(x2), lna1, 'r-', 'LineWidth', 2);
hold on;
plot(real(x2), lna2, 'b-', 'LineWidth', 2);
% b) analytical model
x2r = real(x2);
lna1_an = R*T*log(1-x2r) + x2r.^2.*(1000 + 4000*(2*x2r-1));
lna2_an = R*T*log(x2r) + (1-x2r).^2.*(1000 + 2*4000*x2r);
plot(x2r, lna1_an, 'ro', 'LineWidth', 2);
plot(x2r, lna2_an, 'bo', 'LineWidth', 2);
hold off;
xlabel('x_2', 'FontSize', 14);
ylabel('ln a_i', 'FontSize', 14);
end